"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@code-engine/utils");
const ono_1 = require("ono");
const path = require("path");
const clone_1 = require("../clone/clone");
const clone_context_1 = require("../clone/clone-context");
const clone_file_1 = require("../clone/clone-file");
const await_online_1 = require("./await-online");
const messenger_1 = require("./messenger");
const workerScript = path.join(__dirname, "../worker-thread/index.js");
/**
 * Controls an `Executor` instance running on a worker thread.
 * @internal
 */
class Worker extends messenger_1.Messenger {
    constructor(context) {
        super(workerScript);
        this._context = context;
        this._isTerminated = false;
        this._waitUntilOnline = await_online_1.awaitOnline(this);
        this.on("online", this._handleOnline);
        this.on("exit", this._handleExit);
    }
    /**
     * Loads the specified JavaScript module in the worker thread.
     */
    async loadModule(module, moduleUID) {
        await this._waitUntilOnline;
        this._debug(`CodeEngine worker #${this.threadId} is loading ${module.moduleId}`, { moduleId: module.moduleId });
        let reply = await this.postMessageAsync({
            type: "loadModule",
            cwd: this._context.cwd,
            moduleUID,
            moduleId: module.moduleId,
            data: clone_1.clone(module.data),
        });
        // Return the name of the module's export
        return reply.name;
    }
    /**
     * Processes the given files in the worker thread.
     */
    async *processFile(moduleUID, file, context) {
        await this._waitUntilOnline;
        this._debug(`CodeEngine worker #${this.threadId} is processing ${file}`, { path: file.path });
        let replies = this.postMessageWithReplies({
            type: "processFile",
            moduleUID,
            file: clone_file_1.cloneFile(file),
            context: clone_context_1.cloneContext(context),
        });
        for await (let reply of replies) {
            // tslint:disable-next-line: switch-default
            switch (reply.type) {
                case "log":
                    utils_1.log(context.logger, reply.level, reply.message, reply.data);
                    break;
                case "file":
                    yield reply.file;
                    break;
            }
        }
    }
    /**
     * Terminates the worker thread and cancels all pending operations.
     */
    async terminate() {
        if (this._isTerminated) {
            return 0;
        }
        this._isTerminated = true;
        this.rejectAllPendingMessages(ono_1.ono(`CodeEngine is terminating.`));
        let exitCode = await super.terminate();
        this._debug(`CodeEngine worker #${this.threadId} has terminated`, { exitCode });
        return exitCode;
    }
    /**
     * Logs a debug message when the worker thread comes online.
     */
    _handleOnline() {
        this._debug(`CodeEngine worker #${this.threadId} is online`);
    }
    /**
     * Handles the worker thread exiting, either because we told it to terminate, or because it crashed.
     */
    _handleExit(exitCode) {
        if (!this._isTerminated) {
            // The worker crashed or exited unexpectedly
            this.emit("error", ono_1.ono({ workerId: this.threadId }, `CodeEngine worker #${this.threadId} unexpectedly exited with code ${exitCode}.`));
        }
    }
    /**
     * Logs a debug message for this worker.
     */
    _debug(message, data) {
        this._context.logger.debug(message, { ...data, workerId: this.threadId });
    }
}
exports.Worker = Worker;
//# sourceMappingURL=worker.js.map